use crate::{
    db_connection::DbPool,
    errors::CustomError,
    handlers::pool_handler,
    models::user::{AuthUser, RegisterUser, User},
    utils::jwt::{decode_token, encode_token, UserWithToken},
};
use actix_identity::Identity;
use actix_web::{delete, dev::Payload, post, web, FromRequest, HttpRequest, HttpResponse};
use futures::future::Future;
use std::pin::Pin;

pub type LoggedUser = UserWithToken;

impl FromRequest for LoggedUser {
    type Error = HttpResponse;
    type Config = ();
    type Future = Pin<Box<dyn Future<Output = Result<UserWithToken, HttpResponse>>>>;

    fn from_request(req: &HttpRequest, payload: &mut Payload) -> Self::Future {
        let fut = Identity::from_request(req, payload);

        Box::pin(async move {
            if let Some(identity) = fut.await?.identity() {
                let user = decode_token(&identity)?;
                return Ok(user);
            };
            Err(HttpResponse::Unauthorized().finish())
        })
    }
}

#[post("/auth/register")]
pub async fn register(
    new_user: web::Json<RegisterUser>,
    pool: web::Data<DbPool>,
) -> Result<HttpResponse, HttpResponse> {
    let connection = pool_handler(pool)?;
    let register_user = new_user
        .into_inner()
        .validation()
        .map_err(|e| HttpResponse::InternalServerError().json(e.to_string()))?;

    User::create(register_user, &connection)
        .map(|user| HttpResponse::Ok().json(user))
        .map_err(|e| HttpResponse::InternalServerError().json(e.to_string()))
}

#[post("/auth/login")]
pub async fn login(
    id: Identity,
    auth_user: web::Json<AuthUser>,
    pool: web::Data<DbPool>,
) -> Result<HttpResponse, HttpResponse> {
    let connection = pool_handler(pool)?;

    let user = auth_user.login(&connection).map_err(|e| match e {
        CustomError::DBError(diesel::result::Error::NotFound) => {
            HttpResponse::NotFound().json(e.to_string())
        }
        _ => HttpResponse::InternalServerError().json(e.to_string()),
    })?;

    let token = encode_token(user.id, &user.username, user.admin).map_err(|e| match e {
        _ => HttpResponse::InternalServerError().finish(),
    })?;

    id.remember(String::from(&token));
    Ok(HttpResponse::Ok().json(token))
}

#[delete("/auth/logout")]
pub async fn logout(id: Identity) -> Result<HttpResponse, HttpResponse> {
    id.forget();
    Ok(HttpResponse::Ok().into())
}
